@@ -17,7 +17,7 @@
  */
 
 /* block.cc : block operations */
-/* $Id: block.cc,v 1.12 1997/09/08 18:53:48 lav Exp $ */
+/* $Id: block.cc,v 1.16 1999/10/31 12:12:39 lav Exp $ */
 
 #include <config.h>
 #include <assert.h>
@@ -38,8 +38,6 @@
 
 TextPoint *DragMark=0;
 
-extern const char BlockHelp[];
-
 char    BlockFile[256];
 
 int     hide=TRUE;
@@ -126,11 +124,13 @@ void    RCopy()
 
    ClipBoard cb;
    TextPoint tp=CurrentPos;
+   num old_stdcol=stdcol;
 
    if(!cb.Copy())
       return;
 
    CurrentPos=tp;
+   stdcol=old_stdcol;
    cb.PasteAndMark();
    CurrentPos=BlockBegin;
    stdcol=GetCol();
@@ -338,17 +338,20 @@ void    Write()
    if(hide)
        return;
 
-   if(getstring("Write to: ",BlockFile,sizeof(BlockFile)-1,&LoadHistory,NULL,NULL)<1 || ChooseFileName(BlockFile)<0)
+   if(getstring("Write to: ",BlockFile,sizeof(BlockFile)-1,&LoadHistory,0,
+	 "WriteBlockHelp"," Write Block Help ")<1)
        return;
-   LoadHistory.Push();
-
    if(BlockFile[0]=='|')
    {
+      LoadHistory.Push();
       /* write to pipe */
       Message("Piping to...");
       PipeBlock(BlockFile+1,/*IN*/FALSE,/*OUT*/TRUE);
       return;
    }
+   if(ChooseFileName(BlockFile)<0)
+      return;
+   LoadHistory.Push();
 
    if(stat(BlockFile,&st)==-1)
    {
@@ -378,7 +381,7 @@ void    Write()
            FError(BlockFile);
            return;
        }
-       if(LockFile(fd)==-1)    /* check if the file is already locked */
+       if(LockFile(fd,false)==-1)    /* check if the file is already locked */
        {
            close(fd);
            return;
@@ -490,7 +493,8 @@ void    Read()
 
    if(View)
       return;
-   if(getstring("Read from: ",BlockFile,sizeof(BlockFile)-1,&LoadHistory,NULL,NULL)<1)
+   if(getstring("Read from: ",BlockFile,sizeof(BlockFile)-1,&LoadHistory,0,
+	 "ReadBlockHelp"," Read Block Help ")<1)
       return;
    if(BlockFile[0]=='|')
    {
@@ -746,20 +750,20 @@ next:
       h=TRUE;
 
    if(!h && !View)
-      Message("Block: C-Copy M-Move D-Delete W-Write H-Hide I-Indent U-Unindent R-Read BETXLP");
+      Message("Block: C-Copy M-Move D-Delete W-Write H-Hide I-Indent U-Unindent R-Read BETXLPA");
    else if(h && !View)
-      Message("Block: B-Begin E-End T-Type L-to Lower P-to uPper X-exchange H-display");
+      Message("Block: B-Begin E-End T-Type L-to Lower P-to uPper X-exchange H-display A-mark All");
    else if(h && View)
-      Message("Block: B-Begin E-End T-Type H-display");
+      Message("Block: B-Begin E-End T-Type H-display A-mark All");
    else
-      Message("Block: W-Write B-Begin E-End T-Type H-Hide");
+      Message("Block: W-Write B-Begin E-End T-Type H-Hide A-mark All");
    SetCursor();
    action=GetNextAction();
    flag=REDISPLAY_ALL;
    switch(action)
    {
    case(EDITOR_HELP):
-      Help(BlockHelp," Block Help ");
+      Help("BlockHelp"," Block Help ");
       goto next;
    case(REFRESH_SCREEN):
       UserRefreshScreen();
@@ -831,6 +835,9 @@ next:
       case('Y'):
 	 UserYankBlock();
 	 break;
+      case('A'):
+	 UserMarkAll();
+	 break;
       case('|'):
 #ifndef MSDOS
          UserPipeBlock();
