@@ -1,6 +1,6 @@
-/* 
+/*
  * Copyright (c) 1993-1997 by Alexander V. Lukyanov (lav@yars.free.net)
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -25,9 +25,9 @@
 #define	INVALIDFN	(-4)
 #define	INVALIDNUM	(-5)
 
-extern	sp;
-extern	double	stack[STSIZE];
-extern	calcerrno;
+extern int sp;
+extern double stack[STSIZE];
+extern int calcerrno;
 
 char  *calcerrmsg();
 int   calculator(char *);
