@@ -16,10 +16,11 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: edit.cc,v 1.18 1999/11/06 16:57:54 lav Exp $ */
+
 /* edit.c : main editor loop */
 
 #include <config.h>
-#include <signal.h>
 #include <stdio.h>
 #include <ctype.h>
 #include <fcntl.h>
@@ -37,6 +38,8 @@
 #include "keymap.h"
 #include "block.h"
 #include "screen.h"
+#include "options.h"
+#include "about.h"
 #ifdef WITH_MOUSE
 # include "mouse.h"
 #endif
@@ -65,8 +68,6 @@ int   SaveHst=1;
 
 int   View=FALSE;
 
-int   ShowMatchPos=TRUE;
-
 char  *HOME,*TERM,*DISPLAY;
 
 int   UseColor=1;
@@ -85,19 +86,19 @@ int   getcode()
 {
    long  i;
    static char ch[10];
-   static getcode_active=0;
+   static bool getcode_active=false;
 
    if(getcode_active)
       return(-1);
 
-   getcode_active=1;
+   getcode_active=true;
 
    if(getstring("Char: ",ch,sizeof(ch)-1)<1)
    {
-      getcode_active=0;
+      getcode_active=false;
       return(-1);
    }
-   getcode_active=0;
+   getcode_active=false;
    i=strtol(ch,0,0);
    if(i<0 || i>255)
       return(-1);
@@ -262,6 +263,7 @@ void    Edit()
 	    else
 	       flag|=REDISPLAY_LINE;
 	    ReplaceCharMove(key);
+	    stdcol=GetCol();
 	    continue;
 	 }
 	 if(hex)
@@ -274,6 +276,7 @@ void    Edit()
 	    else
 	    {
 	       ReplaceCharMove(key);
+	       stdcol=GetCol();
 	       flag|=REDISPLAY_LINE;
 	    }
 	    continue;
@@ -366,7 +369,6 @@ void  InitCurses()
    if(le_scr!=NULL)
    {
       delscreen(le_scr);
-      del_curterm(cur_term);
    }
 
    le_scr=newterm(NULL,stdout,stdin);
@@ -393,10 +395,6 @@ void  InitCurses()
 
    idlok(stdscr,useidl);
    scrollok(stdscr,FALSE);
-
-#ifdef WITH_MOUSE
-   mousemask(ALL_MOUSE_EVENTS,0);
-#endif
 }
 void  TermCurses()
 {
@@ -518,6 +516,7 @@ void  PrintUsage(int arg)
 	  "-h  --hex-mode     start in hex mode\n"
 	  "-b  --black-white  black and white mode\n"
 	  "-c  --color        color mode\n"
+	  "    --config=FILE  use specified file instead of le.ini\n"
 	  "    --dump-keymap  dump default keymap to stdout and exit\n"
 	  "    --dump-colors  dump default color map to stdout and exit\n"
 	  "    --mmap         load file using mmap (read only)\n"
@@ -543,6 +542,7 @@ int     main(int argc,char **argv)
       DUMP_COLORS,
       PRINT_HELP,
       PRINT_VERSION,
+      CONFIG_FILE,
       USE_MMAP,
       USE_MMAP_RW
    };
@@ -557,6 +557,7 @@ int     main(int argc,char **argv)
       {"hex-mode",no_argument,0,'h'},
       {"black-white",no_argument,0,'b'},
       {"color",no_argument,0,'c'},
+      {"config",required_argument,0,CONFIG_FILE},
 #ifdef HAVE_MMAP
       {"mmap",no_argument,0,USE_MMAP},
       {"mmap-rw",no_argument,0,USE_MMAP_RW},
@@ -569,11 +570,11 @@ int     main(int argc,char **argv)
 
    strcpy(Program,le_basename(argv[0]));
 
-#ifdef CXX_TYPE_OF_BOOL
-   if(sizeof(bool) != sizeof(CXX_TYPE_OF_BOOL))
+#if defined(CURSES_BOOL) && !defined(bool_redefined)
+   if(sizeof(bool) != sizeof(CURSES_BOOL))
    {
-      fprintf(stderr,"%s: ncurses misconfigured - wrong CXX_TYPE_OF_BOOL\n",Program);
-      exit(1);
+      fprintf(stderr,"%s: warning: curses library has wrong bool type. Expect trouble.\n",Program);
+      sleep(2);
    }
 #endif
 
@@ -605,6 +606,9 @@ int     main(int argc,char **argv)
       {
       case('r'):
          optView=1;
+#ifdef HAVE_MMAP
+	 opt_use_mmap=1;
+#endif
          break;
       case('h'):
          opteditmode=HEXM;
@@ -629,6 +633,7 @@ int     main(int argc,char **argv)
 	 exit(0);
       case(PRINT_VERSION):
 	 printf("%s - %s\n",Program,version_string);
+	 puts(copyright);
 	 exit(0);
       case(USE_MMAP):
 	 opt_use_mmap=1;
@@ -636,12 +641,18 @@ int     main(int argc,char **argv)
 	    optView=2;
 	 else
 	    optView|=2;
+	 opteditmode=HEXM;
 	 break;
       case(USE_MMAP_RW):
 	 opt_use_mmap=1;
 	 if(optView!=-1)
 	    optView&=~2;
+	 opteditmode=HEXM;
       	 break;
+      case(CONFIG_FILE):
+	 ExplicitInitName=true;
+	 strncpy(InitName,optarg,sizeof(InitName)-1);
+	 break;
       }
    }
    if(optUseColor!=-1)
@@ -656,11 +667,7 @@ int     main(int argc,char **argv)
    if(optUseColor!=-1)
       UseColor=optUseColor;
    if(opt_use_mmap!=-1)
-   {
       buffer_mmapped=opt_use_mmap;
-      if(opteditmode==-1)
-	 editmode=HEXM;
-   }
 
    if(optind<argc-1 && argv[optind][0]=='+' && isdigit(argv[optind][1]))
    {
