@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: kern.cc,v 1.8 1997/12/01 12:53:13 lav Exp $ */
+/* $Id: kern.cc,v 1.12 1999/08/25 23:30:27 lav Exp $ */
 
 #include <config.h>
 #include   <stdio.h>
@@ -50,7 +50,7 @@ int       oldmodified;	/* status of text if ptr1 and ptr2 restored */
 num       stdcol;
 num       ScrShift=0;
 int       modified=0;
-int       newfile=0;
+bool      newfile=false;
 
 int       DosEol=0;
 int       EolSize=1;
@@ -65,7 +65,6 @@ mode_t   FileMode;
 int       file=(-1);  /* file descriptor */
 
 char      FileName[256];
-char      InitName[256];
 
 int       TabSize=8;
 int       autoindent=TRUE;
@@ -547,6 +546,36 @@ int   ReadBlockOver(int fd,num size,num *act_read)
    return ERR;
 }
 
+int   GetBlock(char *copy,offs from,num size)
+{
+   if(from<0)
+   {
+     size+=from;
+     from=0;
+   }
+   if(from+size>Size())
+   {
+     size=Size()-from;
+   }
+   if(size<=0)
+      return 0;
+
+   if(from>=ptr1)
+   {
+      memcpy(copy,buffer+from+ptr2-ptr1,size);
+      return(size);
+   }
+   if(from+size<=ptr1)
+   {
+      memcpy(copy,buffer+from,size);
+      return(size);
+   }
+   num leftsize=ptr1-from;
+   memcpy(copy,buffer+from,leftsize);
+   memcpy(copy+leftsize,buffer+ptr2,size-leftsize);
+   return(size);
+}
+
 int   WriteBlock(int fd,offs from,num size,num *act_written)
 {
    num   leftsize;
@@ -943,6 +972,9 @@ void  EmptyText()
 
 void   Optimize()
 {
+   if(View || buffer_mmapped)
+      return;
+
    offs     ptr;
    TextPoint  tp=CurrentPos;
 
