@@ -33,7 +33,9 @@ ActionCodeRec  DefaultActionCodeTable[]=
    {CHAR_LEFT,"$kcub1"},
    {CHAR_RIGHT,"$kcuf1"},
    {WORD_LEFT,"\033|$kcub1"},
+   {WORD_LEFT,"$kLFT"},
    {WORD_RIGHT,"\033|$kcuf1"},
+   {WORD_RIGHT,"$kRIT"},
    {LINE_BEGIN,"$kbeg"},
    {LINE_BEGIN,"$khome"},
    {LINE_BEGIN,"$ka1"},
@@ -42,11 +44,12 @@ ActionCodeRec  DefaultActionCodeTable[]=
    {LINE_BEGIN,"\033|A"},
    {LINE_END,"$kend"},
    {LINE_END,"$kc1"},
+   {LINE_END,"$kll"},
    {LINE_END,"\033|\\$"},
    {LINE_END,"\033|e"},
    {LINE_END,"\033|E"},
    {TEXT_BEGIN,"$kBEG"},
-   {TEXT_BEGIN,"$kHOME"},
+   {TEXT_BEGIN,"$kHOM"},
    {TEXT_BEGIN,"\033|$kbeg"},
    {TEXT_BEGIN,"\033|$khome"},
    {TEXT_BEGIN,"\007|^B"},
@@ -59,12 +62,16 @@ ActionCodeRec  DefaultActionCodeTable[]=
    {TEXT_END,"\007|E"},
    {NEXT_PAGE,"$knp"},
    {NEXT_PAGE,"$kc3"},
+   {NEXT_PAGE,"$knxt"},
    {NEXT_PAGE,"\033|$kcud1"},
    {PREV_PAGE,"$kpp"},
    {PREV_PAGE,"$ka3"},
+   {PREV_PAGE,"$kprv"},
    {PREV_PAGE,"\033|$kcuu1"},
    {PAGE_TOP,"\033|$kpp"},
+   {PAGE_TOP,"$kPRV"},
    {PAGE_BOTTOM,"\033|$knp"},
+   {PAGE_BOTTOM,"$kNXT"},
    {TO_LINE_NUMBER,"\007|\007"},
    {TO_LINE_NUMBER,"\007|g"},
    {TO_LINE_NUMBER,"\007|G"},
@@ -81,6 +88,22 @@ ActionCodeRec  DefaultActionCodeTable[]=
    {LINE_UP,"$kcuu1"},
    {LINE_DOWN,"$kcud1"},
 
+// Movement with block marking (new xterm codes)
+   {MARK_CHAR_LEFT,"\eO2D"},
+   {MARK_CHAR_RIGHT,"\eO2C"},
+   {MARK_WORD_LEFT,"\eO6D"},
+   {MARK_WORD_RIGHT,"\eO6C"},
+   {MARK_LINE_BEGIN,"\eO2H"},
+   {MARK_LINE_END,"\eO2F"},
+   {MARK_TEXT_BEGIN,"\eO6H"},
+   {MARK_TEXT_END,"\eO6F"},
+   {MARK_NEXT_PAGE,"\e[6;2~"},
+   {MARK_PREV_PAGE,"\e[5;2~"},
+   {MARK_PAGE_TOP,"\e[5;6~"},
+   {MARK_PAGE_BOTTOM,"\e[6;6~"},
+   {MARK_LINE_UP,"\eO2A"},
+   {MARK_LINE_DOWN,"\eO2B"},
+
 // Delete actions
    {BACKSPACE_CHAR,"^H"},
    {DELETE_CHAR,"^D"},
@@ -88,12 +111,13 @@ ActionCodeRec  DefaultActionCodeTable[]=
    {BACKSPACE_CHAR,"$kbs"},
    {DELETE_CHAR,"$kdch1"},
    {FORWARD_DELETE_WORD,"\033|$kdch1"},
+   {FORWARD_DELETE_WORD,"$kDC"},
    {DELETE_WORD,"^W"},
    {BACKWARD_DELETE_WORD,"\033|$kbs"},
    {FORWARD_DELETE_WORD,"\033|\177"},
    {BACKWARD_DELETE_WORD,"\033|^H"},
    {DELETE_TO_EOL,"\013"},
-   {DELETE_TO_EOL,"$keol"},
+   {DELETE_TO_EOL,"$kel"},
    {DELETE_LINE,"$kdl"},
    {DELETE_LINE,"\031"},
    {UNDELETE,"^U"},
@@ -121,7 +145,8 @@ ActionCodeRec  DefaultActionCodeTable[]=
 // Block ops
    {COPY_BLOCK,"$kf11"},
    {MOVE_BLOCK,"$kf12"},
-//   {DELETE_BLOCK,"delete-block"},
+   {DELETE_BLOCK,"$1kf12"},
+   {DELETE_BLOCK,"\e$kf12"},
    {SET_BLOCK_END,"$kf6"},
    {SET_BLOCK_END,"\033|6"},
    {SET_BLOCK_BEGIN,"$kf5"},
@@ -143,6 +168,8 @@ ActionCodeRec  DefaultActionCodeTable[]=
    {MARK_LINE,"$1kf5"},
    {MARK_TO_EOL,"\033|$kf6"},
    {MARK_TO_EOL,"$1kf6"},
+   {YANK_BLOCK,"$1kf11"},
+   {YANK_BLOCK,"\e$kf11"},
 
 // Search
    {SEARCH_FORWARD,"$kf7"},
@@ -162,8 +189,11 @@ ActionCodeRec  DefaultActionCodeTable[]=
 //   {FORMAT_ONE_PARA,FormatPara,"format-paragraph"},
 //   {FORMAT_DOCUMENT,FormatAll,"format-document"},
 //   {CENTER_LINE,CenterLine,"center-line"},
+//   {AJUST_RIGHT_LINE,ShiftRightLine,"ajust-right-line"}
    {FORMAT_FUNC_BAR,"\033|$kf4"},
    {FORMAT_FUNC_BAR,"$1kf4"},
+   {FORMAT_FUNC_BAR,"\033|f"},
+   {FORMAT_FUNC_BAR,"\033|F"},
 
 // Others
 //   {CALCULATOR,editcalc,"calculator"},
