@@ -23,9 +23,11 @@
 #include "block.h"
 #include "options.h"
 #include "keymap.h"
+#include "format.h"
 
 ActionProcRec  EditorActionProcTable[]=
 {
+// Movement
    {CHAR_LEFT,UserCharLeft},
    {CHAR_RIGHT,UserCharRight},
    {WORD_LEFT,UserWordLeft},
@@ -44,6 +46,22 @@ ActionProcRec  EditorActionProcTable[]=
    {LINE_UP,UserLineUp},
    {LINE_DOWN,UserLineDown},
 
+// Movement with block marking
+   {MARK_CHAR_LEFT,UserMarkCharLeft},
+   {MARK_CHAR_RIGHT,UserMarkCharRight},
+   {MARK_WORD_LEFT,UserMarkWordLeft},
+   {MARK_WORD_RIGHT,UserMarkWordRight},
+   {MARK_LINE_BEGIN,UserMarkLineBegin},
+   {MARK_LINE_END,UserMarkLineEnd},
+   {MARK_TEXT_BEGIN,UserMarkFileBegin},
+   {MARK_TEXT_END,UserMarkFileEnd},
+   {MARK_NEXT_PAGE,UserMarkPageDown},
+   {MARK_PREV_PAGE,UserMarkPageUp},
+   {MARK_PAGE_TOP,UserMarkPageTop},
+   {MARK_PAGE_BOTTOM,UserMarkPageBottom},
+   {MARK_LINE_UP,UserMarkLineUp},
+   {MARK_LINE_DOWN,UserMarkLineDown},
+
 // Delete actions
    {DELETE_CHAR,UserDeleteChar},
    {BACKSPACE_CHAR,UserBackSpace},
@@ -88,6 +106,8 @@ ActionProcRec  EditorActionProcTable[]=
    {BLOCK_FUNC_BAR,BlockFunc},
    {MARK_LINE,UserMarkLine},
    {MARK_TO_EOL,UserMarkToEol},
+   {START_DRAG_MARK,UserStartDragMark},
+   {YANK_BLOCK,UserYankBlock},
 
 // Search
    {SEARCH_FORWARD,StartSearch},
@@ -102,6 +122,7 @@ ActionProcRec  EditorActionProcTable[]=
    {FORMAT_ONE_PARA,FormatPara},
    {FORMAT_DOCUMENT,FormatAll},
    {CENTER_LINE,CenterLine},
+   {AJUST_RIGHT_LINE,ShiftRightLine},
    {FORMAT_FUNC_BAR,FormatFunc},
 
 // Others
@@ -190,14 +211,3 @@ void  EditorReadKeymap()
 
    fclose(f);
 }
-
-
-/*   char  filename[1024];
-
-   errno=0;
-   sprintf(filename,"%s/.le/keymap-%s",HOME,TERM);
-   f=fopen(filename,"w");
-   if(f==NULL)
-      return;
-
-   fclose(f);*/
