@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: loadsave.cc,v 1.9 1997/12/18 16:32:20 lav Exp $ */
+/* $Id: loadsave.cc,v 1.12 1998/11/26 15:49:02 lav Exp $ */
 
 #include <config.h>
 #include <sys/types.h>
@@ -46,7 +46,7 @@
 #endif
 
 #ifndef __MSDOS__
-int    LockFile(int fd)
+int    LockFile(int fd,bool drop)
 {
    struct  flock   Lock;
    Lock.l_start=0;
@@ -93,7 +93,7 @@ int    LockFile(int fd)
             errno=EACCES;
             while(fcntl(fd,F_SETLK,&Lock)==-1 && (errno==EACCES || errno==EAGAIN))
             {
-	       if(WaitForKey(1000)==OK)
+	       if(WaitForKey(1000)!=ERR)
 	       {
 	       	  int action=GetNextAction();
 		  if(action==CANCEL)
@@ -116,12 +116,17 @@ int    LockFile(int fd)
          return(-2);
       }
    }
+   if(drop)
+   {
+      Lock.l_type=F_RDLCK;	// drop write lock to read lock
+      if(fcntl(fd,F_SETLK,&Lock)==-1)
+	 FError(FileName);
+   }
    return(0);
 }
 #else /* __MSDOS__ */
-int   LockFile(int f)
+int   LockFile(int,bool)
 {
-   (void)f;
    return 0;
 }
 #endif /* __MSDOS__ */
@@ -230,11 +235,11 @@ int   LoadFile(char *name)
       if(S_ISDIR(FileMode))
 	 View|=2;
    }
-   file=open(name,(View?O_RDONLY:O_RDWR|O_CREAT)|O_NDELAY,0644);
+   file=open(name,(View?O_RDONLY:O_RDWR|O_CREAT),0644);
    if(file==-1)
    {
       View|=2;
-      file=open(name,O_RDONLY|O_NDELAY);
+      file=open(name,O_RDONLY);
 	 /* try to open the file in read-only mode */
       if(file==-1)
       {
@@ -250,7 +255,7 @@ int   LoadFile(char *name)
 
    if(!View)
    {
-      int lock_res=LockFile(file);
+      int lock_res=LockFile(file,true);
       if(lock_res==-1)
       {
 	 View&=~2;
@@ -520,16 +525,19 @@ int CheckMode(mode_t mode)
 
 int   SaveFile(char *name)
 {
-   if(buffer_mmapped)
-      return OK;
-
    struct stat st;
    char  msg[256];
    int   nfile;
    num   act_written;
    int   delete_old_file=0;
 
-   if(Text)
+   if(buffer_mmapped)
+   {
+      if(!strcmp(name,FileName))
+	 return OK;
+   }
+
+   if(Text && !View)
       Optimize();
 
    sprintf(msg,"Saving the file \"%s\"...",name);
@@ -540,13 +548,16 @@ int   SaveFile(char *name)
       if(!CheckMode(st.st_mode))
          return(ERR);
 
-     InodeInfo   NewFileInfo(&st,GetLine(),GetCol());
+      InodeInfo   NewFileInfo(&st,GetLine(),GetCol());
 
-     if(file!=-1)
-     {
-        if(FileInfo.SameFileModified(NewFileInfo))
-        {
-           switch(ReadMenuBox(ConCan4Menu,HORIZ,"The file was changed out of the editor",
+      if(file!=-1)
+      {
+	 if(buffer_mmapped && FileInfo.SameFile(NewFileInfo))
+	    return OK;
+
+	 if(FileInfo.SameFileModified(NewFileInfo))
+	 {
+	    switch(ReadMenuBox(ConCan4Menu,HORIZ,"The file was changed out of the editor",
 		     " Warning ",VERIFY_WIN_ATTR,CURR_BUTTON_ATTR))
 	    {
 	    case('C'):
@@ -600,14 +611,14 @@ int   SaveFile(char *name)
    Message(msg);
 
    errno=0;
-   nfile=open(name,O_CREAT|O_RDWR|O_TRUNC,st.st_mode);
+   nfile=open(name,O_CREAT|O_RDWR,st.st_mode);
    if(nfile==-1)
    {
      FError(name);
      return(ERR);
    }
 
-   int lock_res=LockFile(nfile);
+   int lock_res=LockFile(nfile,false);
    if(lock_res==-1)
    {
      close(nfile);
@@ -616,36 +627,34 @@ int   SaveFile(char *name)
    if(lock_res==-2)
       ErrMsg("Warning: file locking failed");
 
-   if(LockEnforce(FileMode))
-   {
-     close(nfile);
-     nfile=open(name,O_CREAT|O_TRUNC|O_RDWR,st.st_mode);
-     if(nfile==-1)
-     {
-       FError(name);
-       return(ERR);
-     }
-     LockFile(nfile);
-   }
-   else
-     close(open(name,O_TRUNC|O_RDONLY));
+   // now after locking truncate the file
+   close(open(name,O_TRUNC|O_RDONLY));
 
    /* force new file to be the same mode as source one */
    chmod(name,st.st_mode);
 
    /* now, after all that stuff, write the buffer contents */
+   errno=0;
    if(WriteBlock(nfile,0,Size(),&act_written)!=OK)
    {
      if(errno)
        FError(name);
+     close(nfile);
      return(ERR);
    }
    if(act_written!=Size())
    {
-     ErrMsg("Cannot write the file up to end\nPerhaps disk full");
+     ErrMsg("Cannot write the file up to end\nPerhaps disk is full");
+     close(nfile);
      return(ERR);
    }
 
+   if(buffer_mmapped)
+   {
+      close(nfile);
+      return OK;
+   }
+
    modified=0;
    CheckPoint();
 
@@ -655,6 +664,7 @@ int   SaveFile(char *name)
 
    close(file);
    file=nfile;
+   LockFile(file,true);
 
    if(delete_old_file)
    {
@@ -704,6 +714,7 @@ int   ReopenRW()
 
    offs oldbb=BlockBegin;
    offs oldbe=BlockEnd;
+   offs oldpos=CurrentPos;
    int oldhide=hide;
 
    int res=LoadFile(name);
@@ -711,6 +722,7 @@ int   ReopenRW()
    {
       BlockBegin=oldbb;
       BlockEnd=oldbe;
+      CurrentPos=oldpos;
       hide=oldhide;
    }
    return res;
