@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: options.cc,v 1.13 1998/01/12 18:32:14 lav Exp $ */
+/* $Id: options.cc,v 1.23 1999/10/31 12:12:40 lav Exp $ */
 
 #include <config.h>
 #include <ctype.h>
@@ -24,6 +24,8 @@
 #include <string.h>
 #include <unistd.h>
 #include <assert.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 #include "edit.h"
 #include "rus.h"
 #include "keymap.h"
@@ -31,15 +33,19 @@
 #include "highli.h"
 #include "getch.h"
 #include "format.h"
+#include "colormnu.h"
+#include "mouse.h"
+
+bool ExplicitInitName=false;
+char InitName[256];
 
-extern char ContextHelpNames[];
 extern int MaxBackup;
 
 extern int le_use_default_colors;
 
 int useidl=0;
 
-extern  grsetno;
+extern int grsetno;
 
 #define STR    0
 #define ONE    1
@@ -49,14 +55,11 @@ extern  grsetno;
 
 int fx,fy;
 
-static   OldTabSize;
+static int OldTabSize;
 
 #define _StrPos 16
 #define _StrLen 49
 
-#define _WinWidth   (_StrPos+_StrLen+3)
-#define _WinHeight  20
-
 struct  opt
 {
    char  *name;
@@ -79,13 +82,15 @@ struct  opt
 {"Save positions",   ONE,  (void*)&SavePos,  22,3},
 
 // {"rectangle &Blocks",ONE,  (void*)&rblock,  45,2},
-{"Make backup",      ONE,  (void*)&makebak,	      45,2},
-{"No regular expr.", ONE,  (void*)&noreg,	      45,3},
-{"Use &colors",      ONE,  (void*)&UseColor,	      45,4},
-{"&Syntax highlight",ONE,  (void*)&hl_option,	      45,5},
-{"Use tabs",         ONE,  (void*)&UseTabs,	      45,6},
-{"BackSp unindents", ONE,  (void*)&BackspaceUnindents,45,7},
-{"Lazy page scroll", ONE,  (void*)&PreferPageTop,     45,8},
+{"No regular expr.", ONE,  (void*)&noreg,	      45,2},
+{"Use &colors",      ONE,  (void*)&UseColor,	      45,3},
+{"&Syntax highlight",ONE,  (void*)&hl_option,	      45,4},
+{"Use tabs",         ONE,  (void*)&UseTabs,	      45,5},
+{"BackSp unindents", ONE,  (void*)&BackspaceUnindents,45,6},
+{"Lazy page scroll", ONE,  (void*)&PreferPageTop,     45,7},
+#ifdef WITH_MOUSE
+{"Use mouse",	     ONE,  (void*)&UseMouse,	      45,8},
+#endif
 
 {"&Latin",           MANY, (void*)&inputmode,	3,5},
 {"r&Ussian",         MANY, (void*)&inputmode,	3,6},
@@ -101,10 +106,10 @@ struct  opt
 {"Vertical scroll",  NUM,  (void*)&Scroll,	22,9},
 {"Horizontal scroll",NUM,  (void*)&hscroll,	22,10},
 
-{"Backup suffix",    STR,  (void*)bak,		17,12,8,sizeof(bak)   },
-{"Backup number",    NUM,  (void*)&MaxBackup,	29,12},
-
-{"backup &Path",     STR,  (void*)BakPath,	17,13,48,sizeof(BakPath)  },
+{"Make backup",      ONE,  (void*)&makebak,	3,12},
+{"Number of backups",NUM,  (void*)&MaxBackup,	22,12},
+{"Backup suffix",    STR,  (void*)bak,		17,13,8,sizeof(bak)   },
+{"backup &Path",     STR,  (void*)BakPath,	17,14,48,sizeof(BakPath)  },
 
 /*
 {"  Save  ",   BUTTON, NULL,   MIDDLE-15,FDOWN-2},
@@ -261,7 +266,7 @@ void  SaveConfToOpenFile(FILE *f,const struct init *init)
    }
 }
 
-void  SaveConfToFile(char *f,const struct init *init)
+void  SaveConfToFile(const char *f,const struct init *init)
 {
    FILE  *conf;
 
@@ -306,13 +311,25 @@ void  fskip(FILE *f)
    while((i=getc(f))!=EOF && i!='\n');
 }
 
-void  ReadConfFromOpenFile(FILE *f,const struct init *init)
+void  ReadConfFromOpenFile(FILE *f,const struct init *init,bool mine)
 {
    const struct init *ptr;
    int    i;
    char  str[256];
    char  *s;
 
+   if(mine)
+   {
+#ifndef __MSDOS__
+      struct stat st;
+      if(fstat(fileno(f),&st)==0)
+      {
+	 if(st.st_uid!=getuid()) // don't read conf from other's files.
+	    return;
+      }
+#endif
+   }
+
    for(;;)
    {
       if(fscanf(f,"%[^=\n]=",str)!=1)
@@ -367,73 +384,96 @@ void  ReadConfFromOpenFile(FILE *f,const struct init *init)
    }
 }
 
-void  ReadConfFromFile(char *ini,const struct init *init)
+void  ReadConfFromFile(const char *ini,const struct init *init,bool mine)
 {
    FILE  *f;
    f=fopen(ini,"r");
    if(f==NULL)
       return;
-   ReadConfFromOpenFile(f,init);
+   ReadConfFromOpenFile(f,init,mine);
    fclose(f);
 }
 
+static bool ConfOK(const char *f,bool mine)
+{
+   if(mine)
+   {
+#ifndef __MSDOS__
+      struct stat st;
+      if(stat(f,&st)==-1)
+	 return false;
+      if(st.st_uid!=getuid())
+	 return false; // don't use other's config
+#endif
+   }
+   if(access(f,R_OK)==-1)
+      return false;
+   return true;
+}
+
 void  ReadConf()
 {
    char  t[256];
 
 #ifndef __MSDOS__
+   bool mine;
+
    sprintf(t,"%s/.le/term-%s",HOME,TERM);
-   if(access(t,R_OK)==-1)
+   if(!ConfOK(t,false))
    {
       sprintf(t,"%s/term-%s",PKGDATADIR,TERM);
-      if(access(t,R_OK)==-1)
+      if(!ConfOK(t,false))
       {
 	 sprintf(t,"%s/.le/term",HOME);
-	 if(access(t,R_OK)==-1)
+	 if(!ConfOK(t,false))
             sprintf(t,"%s/term",PKGDATADIR);
       }
    }
-   ReadConfFromFile(t,term);
+   ReadConfFromFile(t,term,false);
 
    if(chset[0]=='7') // workaround for older version
       init_chset();
 
    sprintf(t,"%s/.le/colors-%s",HOME,TERM);
-   if(access(t,R_OK)==-1)
+   if(!ConfOK(t,false))
    {
       sprintf(t,"%s/colors-%s",PKGDATADIR,TERM);
-      if(access(t,R_OK)==-1)
+      if(!ConfOK(t,false))
       {
 	 sprintf(t,"%s/.le/colors",HOME);
-	 if(access(t,R_OK)==-1)
+	 if(!ConfOK(t,false))
 	    sprintf(t,"%s/colors",PKGDATADIR);
       }
    }
-   ReadConfFromFile(t,colors);
+   ReadConfFromFile(t,colors,false);
    ParseColors();
 
-   strcpy(InitName,".le.ini");
-   if(access(InitName,R_OK)==-1)
+   mine=false;
+   if(!ExplicitInitName)
    {
-      sprintf(InitName,"%s/.le/le.ini",HOME);
-      if(access(InitName,R_OK)==-1)
+      strcpy(InitName,".le.ini");
+      if(!ConfOK(InitName,mine=true))
       {
-	 sprintf(t,"%s/le.ini",PKGDATADIR);
-	 ReadConfFromFile(t,init);
+	 sprintf(InitName,"%s/.le/le.ini",HOME);
+	 if(!ConfOK(InitName,mine=false))
+	 {
+	    sprintf(t,"%s/le.ini",PKGDATADIR);
+	    ReadConfFromFile(t,init,false);
+	    goto ini_done;
+	 }
       }
-      else
-	 ReadConfFromFile(InitName,init);
    }
-   else
-      ReadConfFromFile(InitName,init);
+   ReadConfFromFile(InitName,init,mine);
+
+ini_done:
 
 #else
    sprintf(t,"%s/le-%s",HOME,TERM);
-   ReadConfFromFile(t,term);
+   ReadConfFromFile(t,term,false);
    strcpy(InitName,"le.ini");
    if(access(InitName,R_OK)==-1)
      sprintf(InitName,"%s/le.ini",HOME);
-   ReadConfFromFile(InitName,init);
+   ReadConfFromFile(InitName,init,false);
 #endif
 
    CorrectParameters();
@@ -520,6 +560,10 @@ void  CorrectParameters()
    }
 
    init_attrs();
+
+#ifdef WITH_MOUSE
+   SetupMouse();
+#endif
 }
 
 int GetNo(const struct opt *p,const struct opt *p1)
@@ -577,17 +621,41 @@ int GetDist(const struct opt *to,int action)
    return(d);
 }
 
+#ifdef WITH_MOUSE
+static bool InOptField(int y,int x,struct opt *o)
+{
+   int oy=o->y;
+   int ox=o->x;
+
+   int w=3;
+   if(o->type==NUM)
+   {
+      w=o->len;
+      if(w==0)
+	 w=4;
+   }
+   else if(o->type==STR)
+      w=o->len;
+   else if(o->type==BUTTON)
+      w=ItemLen(o->name);
+
+   Absolute(&ox,w,Upper->w);
+   Absolute(&oy,1,Upper->h);
+
+   return (y==oy && x>=ox && x<ox+w);
+}
+#endif /* WITH_MOUSE */
 
 void  W_Dialogue(struct opt *opt,
              const char *SetupHelp,const char *SetupTitle,
-             int (*EatKey)(int),int (*HandleButton)(char *))
+             int (*EatKey)(int),int (*HandleButton)(char *,int))
 {
    int newitem=0;
    int first=1;
    char  s[512];
    struct opt *p,*p1,*n,*curr=opt;
    int shift=0,pos=0,i,key=0,d,dist;
-   int   action;
+   int   action=-1;
    int   OldShowStatusLine=ShowStatusLine;
    attr  *a=Upper->a;
 
@@ -699,6 +767,29 @@ use_key:
          continue;
       switch(action)
       {
+#ifdef WITH_MOUSE
+      case(MOUSE_ACTION):
+      {
+	 MEVENT mev;
+	 if(getmouse(&mev)==ERR)
+	    continue;
+	 if(!(mev.bstate&ALL_MOUSE_EVENTS))
+	    continue;
+	 for(p=opt; p->name; p++)
+	 {
+	    if(InOptField(mev.y-Upper->y,mev.x-Upper->x,p))
+	    {
+	       curr=p;
+	       if(curr->type==ONE || curr->type==MANY || curr->type==BUTTON)
+	       {
+		  ungetch(' ');
+	       }
+	       continue;
+	    }
+	 }
+	 continue;
+      }
+#endif // WITH_MOUSE
       case(NEWLINE):
       case(CANCEL):
          goto leave_cycle;
@@ -772,6 +863,9 @@ use_key:
             case(MANY):
                *(int*)(curr->var)=GetNo(curr,opt);
                break;
+	    case(BUTTON):
+	       action=HandleButton(curr->name,curr-opt);
+	       goto use_key;
             }
             break;
          }
@@ -899,7 +993,7 @@ use_key:
 leave_cycle:
    if(action==CANCEL)
    {
-esc:  for(p=opt; p->name; p++)
+      for(p=opt; p->name; p++)
       {
          if(p->type==STR)
 	 {
@@ -916,8 +1010,8 @@ esc:  for(p=opt; p->name; p++)
       stdcol=GetCol();
       if(curr->type==BUTTON)
       {
-         if(HandleButton(curr->name)==CANCEL)
-            goto esc;
+         action=HandleButton(curr->name,curr-opt);
+	 goto use_key;
       }
    }
    for(p=opt; p->name; p++)
@@ -935,7 +1029,7 @@ esc:  for(p=opt; p->name; p++)
 
 void  Dialogue(struct opt *opt,int WinWidth,int WinHeight,char *WinTitle,
              const char *SetupHelp,const char *SetupTitle,
-             int (*EatKey)(int),int (*HandleButton)(char *))
+             int (*EatKey)(int),int (*HandleButton)(char *,int))
 {
    WIN *optw;
    optw=CreateWin(MIDDLE,MIDDLE,WinWidth,WinHeight,DIALOGUE_WIN_ATTR,WinTitle,0);
@@ -962,16 +1056,14 @@ int    OptEatKey(int k)
    return(-1);
 }
 
-int    OptHandleBut(char *n)
+int    OptHandleBut(char *,int)
 {
-   (void)n;
    return(0);
 }
 
 void  Options()
 {
-   extern const char OptionsHelp[];
-   Dialogue(opt,68,16," Options ",OptionsHelp," Setup Help ",OptEatKey,OptHandleBut);
+   Dialogue(opt,68,17," Options ","OptionsHelp"," Setup Help ",OptEatKey,OptHandleBut);
 }
 
 void  SaveOpt()
@@ -991,9 +1083,8 @@ int   TOEatKey(int k)
    (void)k;
    return(-1);
 }
-int   TOHandleBut(char *button_name)
+int   TOHandleBut(char *,int)
 {
-   (void)button_name;
    return(0);
 }
 void  TermOpt(void)
@@ -1014,7 +1105,7 @@ void  ProgOpt(void)
    Dialogue(ProgOptTbl,70,9," External Programs ",NULL,NULL,TOEatKey,TOHandleBut);
 }
 
-static bg,fg,c_bold,c_rev,c_ul,c_dim,b_bold,b_rev,b_ul,b_dim;
+static int bg,fg,c_bold,c_rev,c_ul,c_dim,b_bold,b_rev,b_ul,b_dim;
 
 void  EditColor(color *cp,color *bp)
 {
@@ -1088,84 +1179,75 @@ void  EditColor(color *cp,color *bp)
    bp->attr=(b_bold?A_BOLD:0)|(b_rev?A_REVERSE:0)|(b_ul?A_UNDERLINE:0)|(b_dim?A_DIM:0);
 }
 
+static color new_color_pal[MAX_COLOR_NO+1];
+static color new_bw_pal[MAX_COLOR_NO+1];
+static bool color_applied;
+
+int ColorHandleBut(char *button,int index)
+{
+   static int color_xlat[]={
+      NORMAL_TEXT,BLOCK_TEXT,STATUS_LINE,SCROLL_BAR,ERROR_WIN,VERIFY_WIN,
+      HELP_WIN,DIALOGUE_WIN,MENU_WIN,CURR_BUTTON,DISABLED_ITEM,SHADOWED,
+      SYNTAX1,SYNTAX2,SYNTAX3};
+   if(index<(int)(sizeof(color_xlat)/sizeof(*color_xlat)))
+   {
+      int color_no=color_xlat[index];
+      EditColor(FindColor(new_color_pal,color_no),
+                FindColor(new_bw_pal,color_no));
+      return -1;
+   }
+   char *l=strchr(button,'&');
+   if(!l)
+      return -1;
+   char res=toupper(l[1]);
+   if(res=='S' || res=='U' || res=='T' || res=='O')
+   {
+      memcpy(color_pal,new_color_pal,sizeof(new_color_pal));
+      memcpy(bw_pal,new_bw_pal,sizeof(new_bw_pal));
+      init_attrs();
+      color_applied=true;
+   }
+   if(res=='S')
+      ColorsSave();
+   else if(res=='T')
+      ColorsSaveForTerminal();
+   return CANCEL;
+}
+
 void  ColorsOpt()
 {
-   color new_color_pal[MAX_COLOR_NO+1];
-   color new_bw_pal[MAX_COLOR_NO+1];
-
    memcpy(new_color_pal,color_pal,sizeof(new_color_pal));
    memcpy(new_bw_pal,bw_pal,sizeof(new_bw_pal));
 
-   WIN *color_win=CreateWin(MIDDLE,MIDDLE,40,22,MENU_ATTR," Select color to tune ");
-   DisplayWin(color_win);
+   static struct opt m[]={
+      {" Normal text                        ",BUTTON,NULL,2,2},
+      {" Block text                         ",BUTTON,NULL,2,3},
+      {" Status line                        ",BUTTON,NULL,2,4},
+      {" Scroll bar                         ",BUTTON,NULL,2,5},
+      {" Error window                       ",BUTTON,NULL,2,6},
+      {" Verify window                      ",BUTTON,NULL,2,7},
+      {" Help window                        ",BUTTON,NULL,2,8},
+      {" Dialogue window                    ",BUTTON,NULL,2,9},
+      {" Menu                               ",BUTTON,NULL,2,10},
+      {" Current button                     ",BUTTON,NULL,2,11},
+      {" Disabled button                    ",BUTTON,NULL,2,12},
+      {" Shadow                             ",BUTTON,NULL,2,13},
+      {" Syntax 1                           ",BUTTON,NULL,2,14},
+      {" Syntax 2                           ",BUTTON,NULL,2,15},
+      {" Syntax 3                           ",BUTTON,NULL,2,16},
+//       {"[&Save]",			      BUTTON,NULL,MIDDLE-15,FDOWN-2},
+//       {"[Save for &tty]",                     BUTTON,NULL,MIDDLE-4, FDOWN-2},
+//       {"[&Use]",                              BUTTON,NULL,MIDDLE+7, FDOWN-2},
+//       {"[&Cancel]",			      BUTTON,NULL,MIDDLE+14,FDOWN-2},
+      {"[   &Ok   ]",                         BUTTON,NULL,MIDDLE-8,FDOWN-2},
+      {"[ &Cancel ]",			      BUTTON,NULL,MIDDLE+8,FDOWN-2},
+      {NULL}};
 
-   static struct menu m[]=
-   {
-      {"Normal text                         ",2,1},
-      {"Block text                          ",2,2},
-      {"Status line                         ",2,3},
-      {"Scroll bar                          ",2,4},
-      {"Error window                        ",2,5},
-      {"Verify window                       ",2,6},
-      {"Help window                         ",2,7},
-      {"Dialogue window                     ",2,8},
-      {"Menu                                ",2,9},
-      {"Current button                      ",2,10},
-      {"Disabled button                     ",2,11},
-      {"Shadow                              ",2,12},
-      {"Syntax 1                            ",2,13},
-      {"Syntax 2                            ",2,14},
-      {"Syntax 3                            ",2,15},
-      {"---",2,FDOWN-5},
-      {"&Save colors                         ",2,FDOWN-4},
-      {"Save as &terminal specific           ",2,FDOWN-3},
-      {"&Use colors for this session         ",2,FDOWN-2},
-      {"&Cancel                            ^X",2,FDOWN-1},
-      {0}
-   };
+   color_applied=false;
 
-   bool applied=false;
-   int curr=0;
-   for(;;)
-   {
-      int res=ReadMenu(m,VERT,MENU_ATTR,CURR_BUTTON_ATTR,curr);
-      if(res<0)
-      {
-	 curr=-res-1;
-	 static int color_xlat[]={
-	    NORMAL_TEXT,BLOCK_TEXT,STATUS_LINE,SCROLL_BAR,ERROR_WIN,VERIFY_WIN,
-	    HELP_WIN,DIALOGUE_WIN,MENU_WIN,CURR_BUTTON,DISABLED_ITEM,SHADOWED,
-	    SYNTAX1,SYNTAX2,SYNTAX3};
-	 assert(unsigned(curr)<sizeof(color_xlat)/sizeof(color_xlat[0]));
-	 int color_no=color_xlat[curr];
-	 EditColor(FindColor(new_color_pal,color_no),
-		   FindColor(new_bw_pal,color_no));
-	 continue;
-      }
-      if(res=='S' || res=='U' || res=='T')
-      {
-	 memcpy(color_pal,new_color_pal,sizeof(new_color_pal));
-	 memcpy(bw_pal,new_bw_pal,sizeof(new_bw_pal));
-	 init_attrs();
-      	 applied=true;
-      }
-      if(res=='S' || res=='T')
-      {
-	 DescribeColors(bw_pal,color_pal);
-	 const char *const n="/.le/colors";
-	 char *f=(char*)alloca(strlen(HOME)+strlen(n)+1+strlen(TERM)+1);
-	 if(res=='S')
-	    sprintf(f,"%s%s",HOME,n);
-	 else
-	    sprintf(f,"%s%s-%s",HOME,n,TERM);
-	 SaveConfToFile(f,colors);
-      }
-      break;
-   }
+   Dialogue(m,40,21," Select color to tune ",NULL,NULL,TOEatKey,ColorHandleBut);
 
-   CloseWin();
-   DestroyWin(color_win);
-   if(applied)
+   if(color_applied)
    {
       clearok(stdscr,1);
       flag=REDISPLAY_ALL;
