@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: screen.h,v 1.4 1997/10/25 15:30:28 lav Exp $ */
+/* $Id: screen.h,v 1.6 1998/11/26 23:29:50 lav Exp $ */
 
 #ifndef SCREEN_H
 #define SCREEN_H
@@ -32,16 +32,33 @@ void  SetCursor();
 void  SyncTextWin();
 
 void  Message(const char *msg);
+void  AddMessage(const char *msg);
+void  ClearMessage();
+
 void  ErrMsg(const char *msg);
 
 void  TestPosition();
 
-extern WINDOW *text_w;
-extern WINDOW *status_w;
-
 #define   HexPos    11
 #define   AsciiPos   (HexPos+3*16+2)
 
-extern ScrollBarPos;
+extern int ScrollBarPos;
+
+extern int TextWinX,TextWinY,TextWinWidth,TextWinHeight,ScrollBarX,StatusLineY;
+
+extern int ShowScrollBar;
+extern int ShowStatusLine;
+
+extern int ShowMatchPos; // for hex mode
+
+enum {
+    SHOW_RIGHT=0,
+    SHOW_NONE=1,
+    SHOW_LEFT=2,
+    SHOW_BOTTOM=0,
+    SHOW_TOP=2
+};
+
+extern num ScrShift;
 
 #endif//SCREEN_H
