@@ -17,12 +17,13 @@
  */
 
 /* Search & replace for the editor */
-/* $Id: search.cc,v 1.16 1997/09/02 19:01:09 lav Exp $ */
+/* $Id: search.cc,v 1.26 1999/11/06 16:57:54 lav Exp $ */
 
 #include <config.h>
 #include <memory.h>
 #include <ctype.h>
 #include <string.h>
+#include <stdlib.h>
 #include "edit.h"
 #include "keymap.h"
 #include "getch.h"
@@ -56,6 +57,10 @@ static struct re_registers regs;
 static struct re_pattern_buffer rexp;
 static bool rexp_compiled=false;
 static char fastmap[256];
+static bool word_bounds;
+static bool numeric_search;
+static bool hex_search;
+static bool no_regex;
 
 int   LastOp=0;
 int   LastDir=FORWARD;
@@ -79,31 +84,107 @@ void  NotFound()
       ScreenTop=NextLine(ScreenTop);
       flag=REDISPLAY_ALL;
    }
+   CurrentPos=back_tp;
    SyncTextWin();
    StatusLine();
    Message("Search string not found.");
-   CurrentPos=back_tp;
    stdcol=GetCol();
    SetCursor();
    WaitForKey();
 }
 
-int   CompilePattern()
+static unsigned char map_to_lower[256];
+static void map_to_lower_init()
 {
+   if(map_to_lower[' '])  // assumes tolower(' ')!='\0' :-)
+      return;
+
+   unsigned i;
+   /* Map uppercase characters to corresponding lowercase ones.  */
+   for (i = 0; i < sizeof(map_to_lower); i++)
+      map_to_lower[i] = isupper (i) ? tolower (i) : i;
+}
+
+static bool CompilePattern()
+{
+   if(noreg)
+      return true;
+
 //    if(rexp_compiled)
 //       regfree(&rexp);
 
    re_syntax_options=RE_BK_PLUS_QM|RE_CHAR_CLASSES|RE_CONTEXT_INDEP_ANCHORS|
 		     RE_UNMATCHED_RIGHT_PAREN_ORD;
-   const char *err=re_compile_pattern((char*)pattern,patlen,&rexp);
+   rexp.translate=0;
+   word_bounds=false;
+   hex_search=false;
+   numeric_search=false;
+   no_regex=false;
+
+   unsigned char *p=pattern;
+   int len=patlen;
+   if(*p=='$') // search options
+   {
+      for(p++,len--; len>0 && *p!=' '; p++,len--)
+      {
+	 switch(*p)
+	 {
+	    case('i'): // ignore case
+	       map_to_lower_init();
+	       rexp.translate=map_to_lower;
+	       break;
+	    case('w'):
+	       word_bounds=true;
+	       break;
+	    case('n'):
+	       numeric_search=true;
+	       break;
+	    case('x'):
+	       hex_search=true;
+	       break;
+	    default:
+	       goto so_out;
+	 }
+      }
+      if(len>0 && *p==' ')
+	 p++,len--;
+      if(len==0)
+      {
+	 p=pattern;
+	 len=patlen;
+      }
+   so_out:;
+   }
+
+   if(numeric_search || hex_search)
+   {
+      int radix=0;
+      if(hex_search)
+	 radix=16;
+      char *s=(char*)alloca(len+1);
+      char *store=s;
+      len=0;
+      for(char *t=strtok((char*)p," "); t; t=strtok(0," "))
+      {
+	 int num=strtoul(t,0,radix);
+	 unsigned char n=(unsigned char)num;
+	 if(strchr("[]\\*.",n))
+	    *store++='\\', len++;
+	 *store++=n, len++;
+      }
+      *store=0;
+      p=(unsigned char*)s;
+   }
+
+   const char *err=re_compile_pattern((char*)p,len,&rexp);
    if(err)
    {
       ErrMsg(err);
-      return 0;
+      return false;
    }
    rexp_compiled=true;
    rexp.fastmap=fastmap;
-   return 1;
+   return true;
 }
 
 int   no_re_search_2(const char *str,const int slen,
@@ -248,13 +329,19 @@ int    Search(int dir,offs offslim)
       buf1=buf2; buf2=0;
       len1=len2; len2=0;
    }
+search_again:
    int res;
    if(noreg)
       res=no_re_search_2((char*)pattern,patlen,buf1,len1,
 			 buf2,len2,srchpos,offslim-srchpos);
    else
+   {
+      offs stop=len1+len2;
+      if(dir==FORWARD)
+	 stop=offslim;
       res=re_search_2(&rexp,buf1,len1,buf2,len2,
-		      srchpos,offslim-srchpos,&regs,len1+len2);
+		      srchpos,offslim-srchpos,&regs,stop);
+   }
    if(res==-1)
       return FALSE;
 
@@ -264,50 +351,114 @@ int    Search(int dir,offs offslim)
    else
       fndlen=regs.end[0]-res;
    CurrentPos=fndind;
+
+   if(word_bounds)
+   {
+      if(isalnum(CharRel(-1)) || isalnum(CharRel(fndlen)))
+      {
+	 if(dir==FORWARD)
+	    srchpos=CurrentPos+1;
+	 else // dir==BACKWARD
+	    srchpos=CurrentPos-1;
+	 goto search_again;
+      }
+   }
+
    return(TRUE);
 }
 
 void  ReplaceFound()
 {
-   register int  i;
    offs	 o=0;
 
    if(!noreg)
    {
-      for(i=0; i<replen; i++)
+      char *scan=(char*)replace;
+      int scan_len=replen;
+      for(;;)
       {
-         switch(replace[i])
-         {
-         case('\\'):
-	    if(i < (replen-1))
-            {
-	       i++;
-	       char ch=replace[i];
-	       if(ch=='&')
-		  ch='0';
-	       if(ch>='0' && ch<='9')
+	 if(scan_len==0)
+	    break;
+	 char *bslash=(char*)memchr(scan,'\\',scan_len);
+	 int len=(bslash?bslash-scan:scan_len);
+	 if(len==scan_len-1)
+	    len=scan_len;  // backslash at eol
+	 if(len>0)
+	 {
+	    if(buffer_mmapped)
+	    {
+	       ReplaceBlock(scan,len);
+	       CurrentPos+=len;
+	    }
+	    else
+	    {
+	       InsertBlock(scan,len);
+	       o+=len;
+	    }
+	    scan+=len;
+	    scan_len-=len;
+	 }
+	 if(!bslash)
+	    break;
+	 scan++;
+	 scan_len--;
+	 if(scan_len==0)
+	    break;
+	 char ch=*scan++;
+	 scan_len--;
+	 if(ch=='\\')
+	 {
+	 insert_ch:
+	    if(buffer_mmapped)
+	       ReplaceCharMove(ch);
+	    else
+	    {
+	       InsertChar(ch);
+	       o++;
+	    }
+	    continue;
+	 }
+	 if(ch=='&')
+	    ch='0';
+	 if(ch>='0' && ch<='9')
+	 {
+	    unsigned n=ch-'0';
+	    if(n<regs.num_regs)
+	    {
+	       if(!buffer_mmapped)
 	       {
-		  unsigned n=ch-'0';
-		  if(n<regs.num_regs)
-		  {
-		     CopyBlock(regs.start[n]+o,regs.end[n]-regs.start[n]);
-		     o+=regs.end[n]-regs.start[n];
-		  }
-		  break;
+		  CopyBlock(regs.start[n]+o,regs.end[n]-regs.start[n]);
+		  o+=regs.end[n]-regs.start[n];
+	       }
+	       else
+	       {
+		  int rlen=regs.end[n]-regs.start[n];
+		  ReplaceBlock(buffer+regs.start[n],rlen);
+		  CurrentPos+=rlen;
 	       }
 	    }
-         default:
-            InsertChar(replace[i]);
-            o++;
-	    break;
-         }
+	    continue;
+	 }
+	 // we've got an unknown backslashed sequence. Insert it verbatim.
+	 scan--;
+	 scan_len++;
+	 ch='\\';
+	 goto insert_ch;
       }
    }
    else
    {
-      InsertBlock((char*)replace,replen);
+      if(buffer_mmapped)
+      {
+	 ReplaceBlock((char*)replace,replen);
+	 CurrentPos+=replen;
+      }
+      else
+	 InsertBlock((char*)replace,replen);
    }
-   DeleteBlock(0,fndlen);   /* delete found substitute */
+   if(!buffer_mmapped)
+	DeleteBlock(0,fndlen);   /* delete found substitute */
+   flag=REDISPLAY_ALL;
 }
 
 void  Replace()
@@ -324,9 +475,18 @@ void  Replace()
 
    back_tp=CurrentPos;
 
+   TextPoint block_end=(rblock
+      ?(BlockEnd.Col()==BlockBegin.Col()?NextLine(BlockEnd):BlockEnd.Offset())
+      :BlockEnd.Offset());
+
    do
    {
-      if(!Search(FORWARD,(key!='#'?TextEnd:BlockEnd)))
+      if(key!='*' && key!='#')
+      {
+	 StatusLine();
+	 SyncTextWin(); // before possible long search
+      }
+      if(!Search(FORWARD,(key!='#'?TextEnd:block_end)))
       {
          if(first)
             NotFound();
@@ -375,7 +535,17 @@ void  Replace()
          hide=oldh;
          Message("Replace? Y,R-replace L-Last replace N,D-do not replace *-replace all #-in block");
          SetCursor();
+      next_action:
          action=GetNextAction();
+	 switch(action)
+	 {
+	 case REFRESH_SCREEN:
+	    refresh();
+	    goto next_action;
+	 default:
+	    if(StringTypedLen!=1)
+	       goto ret;
+	 }
          key=toupper((byte)(StringTyped[0]));
       }
       if(key=='Y' || key=='R' || action==NEWLINE || key=='*' || key=='L')
@@ -513,7 +683,7 @@ void  ContSearch()
 void  StartSearch()
 {
    if(getstring("Search forwards: ",(char*)pattern,sizeof(pattern)-1,
-                                  &SearchHistory,&patlen,NULL)<1)
+                                  &SearchHistory,&patlen,"SearchHelp"," Search Help ")<1)
      return;
    LastDir=FORWARD;
    LastOp=SEARCH;
@@ -524,7 +694,7 @@ void  StartSearch()
 void  StartSearchBackward()
 {
    if(getstring("Search backwards: ",(char*)pattern,sizeof(pattern)-1,
-                                  &SearchHistory,&patlen,NULL)<1)
+                                  &SearchHistory,&patlen,"SearchHelp"," Search Help ")<1)
      return;
    LastDir=BACKWARD;
    LastOp=SEARCH;
@@ -547,9 +717,9 @@ void  StartReplace()
 {
    if(View)
       return;
-   if(getstring("Search: ",(char*)pattern,sizeof(pattern)-1,&SearchHistory,&patlen,NULL)<1)
+   if(getstring("Search: ",(char*)pattern,sizeof(pattern)-1,&SearchHistory,&patlen,"SearchHelp"," Search Help ")<1)
       return;
-   if(getstring("Replace: ",(char*)replace,sizeof(replace)-1,&SearchHistory,&replen,NULL)<0)
+   if(getstring("Replace: ",(char*)replace,sizeof(replace)-1,&SearchHistory,&replen,"ReplaceHelp"," Replace Help ")<0)
       return;
    LastOp=REPLACE;
    if(!CompilePattern())
