@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: user.cc,v 1.11 1997/12/04 16:41:24 lav Exp $ */
+/* $Id: user.cc,v 1.21 1999/11/06 16:57:54 lav Exp $ */
 
 #include <config.h>
 #include <ctype.h>
@@ -34,6 +34,7 @@
 #include "clipbrd.h"
 #include "getch.h"
 #include "format.h"
+#include "about.h"
 
 void  UserDeleteToEol()
 {
@@ -568,12 +569,18 @@ void  UserSetBlockBegin()
    if(rblock?(CurrentPos.Line()<=BlockEnd.Line()
               && CurrentPos.Col()<=BlockEnd.Col())
             :(CurrentPos.Offset()<=BlockEnd.Offset()))
+   //then
       BlockBegin=CurrentPos;
    else
    {
       BlockBegin=/*BlockEnd;*/
       BlockEnd=CurrentPos;
    }
+   if(DragMark)
+   {
+      if(*DragMark < BlockBegin)
+	 *DragMark = BlockBegin;
+   }
 }
 void  UserSetBlockEnd()
 {
@@ -588,13 +595,18 @@ void  UserSetBlockEnd()
    if(rblock?(CurrentPos.Line()>=BlockBegin.Line()
               && CurrentPos.Col()>=BlockBegin.Col())
             :(CurrentPos.Offset()>=BlockBegin.Offset()))
-   /*then*/
+   //then
       BlockEnd=CurrentPos;
    else
    {
       BlockEnd=/*BlockBegin;*/
       BlockBegin=CurrentPos;
    }
+   if(DragMark)
+   {
+      if(*DragMark > BlockEnd)
+	 *DragMark = BlockEnd;
+   }
 }
 
 void  UserFindBlockBegin()
@@ -690,7 +702,10 @@ void  UserUnindent()
       if(Text && Eol())
       {
          DeleteBlock(CurrentPos-LineBegin(CurrentPos),0);
-         stdcol=newmargin;
+         if(newmargin<curpos)
+	    stdcol=newmargin;
+	 else
+	    stdcol=0;
 	 flag|=REDISPLAY_LINE;
          return;
       }
@@ -721,7 +736,7 @@ void  UserBackSpace()
 {
    if(View)
       return;
-   if(Bof())
+   if(Bof() && (!Text || stdcol==0))
       return;
    if(hex)
    {
@@ -998,6 +1013,7 @@ void  UserToOffset()
 
 void  UserIndent()
 {
+   /* #### what exactly needs to be done when !insert ? */
    if(Text && stdcol>=GetCol() && Eol())
    {
       stdcol=(stdcol/IndentSize+1)*IndentSize;
@@ -1024,8 +1040,11 @@ void  UserIndent()
       return;
    }
    PreUserEdit();
-   while(!Bol() && (CharRel(-1)==' ' || CharRel(-1)=='\t'))
-      BackSpace();
+   if(insert)
+   {
+      while(!Bol() && (CharRel(-1)==' ' || CharRel(-1)=='\t'))
+	 BackSpace();
+   }
    while(GetCol()<newcol)
    {
       if(insert || Eol())
@@ -1206,7 +1225,7 @@ void  UserEnterControlChar()
    if(View)
       return;
 
-   attrset(STATUS_LINE_ATTR->attr);
+   attrset(STATUS_LINE_ATTR->n_attr);
    mvaddch(StatusLineY,COLS-2,'^');
    SetCursor();
    key=GetRawKey();
@@ -1223,8 +1242,7 @@ void  UserWordHelp()
 
 void  UserKeysHelp()
 {
-   extern const char MainHelp[];
-   Help(MainHelp," Help on Keys ");
+   Help("MainHelp"," Help on Keys ");
 }
 
 void  UserAbout()
@@ -1257,7 +1275,7 @@ void  UserInsertCharCode()
       UserInsertControlChar(ch);
 }
 
-static base_editmode=-1;
+static int base_editmode=-1;
 
 void  UserSwitchInsertMode()
 {
@@ -1368,8 +1386,11 @@ void  UserPipeBlock()
 
 void  UserYankBlock()
 {
+   if(View)
+      return;
    MainClipBoard.PasteAndMark();
    OptionallyConvertBlockNewLines("yanked");
+   flag=REDISPLAY_ALL;
 }
 
 void  UserStartDragMark()
@@ -1379,6 +1400,7 @@ void  UserStartDragMark()
       UserStopDragMark();
       return;
    }
+   PreUserEdit();
    DragMark=new TextPoint(CurrentPos);
    if(hide)
       UserSetBlockBegin();
@@ -1390,3 +1412,114 @@ void  UserStopDragMark()
    delete DragMark;
    DragMark=0;
 }
+
+static TextPoint *mark_move_point;
+static bool mark_move_top,mark_move_left;
+static void pre_mark_move()
+{
+   if(hide)
+   {
+   was_hidden:
+      flag=REDISPLAY_ALL;
+      BlockEnd=BlockBegin=CurrentPos;
+      mark_move_point=&BlockEnd;
+      mark_move_top=mark_move_left=false;
+      return;
+   }
+   if(rblock)
+   {
+      mark_move_top=(CurrentPos.Line()==BlockBegin.Line());
+      if(!mark_move_top && CurrentPos.Line()!=BlockEnd.Line())
+	 goto was_hidden;
+      mark_move_left=(CurrentPos.Col()==BlockBegin.Col());
+      if(!mark_move_left && CurrentPos.Col()!=BlockEnd.Col())
+	 goto was_hidden;
+   }
+   else
+   {
+      if(CurrentPos==BlockBegin)
+	 mark_move_point=&BlockBegin;
+      else if(CurrentPos==BlockEnd)
+	 mark_move_point=&BlockEnd;
+      else
+	 goto was_hidden;  // just do the same.
+   }
+}
+static void post_mark_move()
+{
+   hide=false;
+   PreUserEdit();
+   if(rblock)
+   {
+      if(mark_move_left)
+      {
+	 if(mark_move_top)
+	    BlockBegin=CurrentPos;
+	 else
+	 {
+	    BlockBegin=TextPoint::ForcedLineCol(BlockBegin.Line(),CurrentPos.Col());
+	    BlockEnd  =TextPoint::ForcedLineCol(CurrentPos.Line(),BlockEnd.Col());
+	 }
+      }
+      else
+      {
+	 if(mark_move_top)
+	 {
+	    BlockBegin=TextPoint::ForcedLineCol(CurrentPos.Line(),BlockBegin.Col());
+	    BlockEnd  =TextPoint::ForcedLineCol(BlockEnd.Line(),CurrentPos.Col());
+	 }
+	 else
+	    BlockEnd=CurrentPos;
+      }
+      // swap the points if needed.
+      if(BlockBegin.Col()>BlockEnd.Col() && BlockBegin.Line()>=BlockEnd.Line())
+	 goto swap_marks;
+      if(BlockBegin.Col()>BlockEnd.Col())
+      {
+	 TextPoint tmp=BlockBegin;
+	 BlockBegin=TextPoint::ForcedLineCol(BlockBegin.Line(),BlockEnd.Col());
+	 BlockEnd  =TextPoint::ForcedLineCol(BlockEnd.Line(),tmp.Col());
+      }
+      else if(BlockBegin.Line()>BlockEnd.Line())
+      {
+	 TextPoint tmp=BlockBegin;
+	 BlockBegin=TextPoint::ForcedLineCol(BlockEnd.Line(),BlockBegin.Col());
+	 BlockEnd  =TextPoint::ForcedLineCol(tmp.Line(),BlockEnd.Col());
+      }
+   }
+   else
+   {
+      if(*mark_move_point!=CurrentPos)
+	 flag=REDISPLAY_ALL;
+      *mark_move_point=CurrentPos;
+      if(BlockBegin>BlockEnd)
+      {
+      swap_marks:
+	 TextPoint tmp=BlockBegin;
+	 BlockBegin=BlockEnd;
+	 BlockEnd=tmp;
+      }
+   }
+}
+
+#define MarkMove(move)	   \
+   void UserMark##move()   \
+   {			   \
+      pre_mark_move();	   \
+      User##move();	   \
+      post_mark_move();	   \
+   }
+MarkMove(CharLeft);
+MarkMove(CharRight);
+MarkMove(WordLeft);
+MarkMove(WordRight);
+MarkMove(LineBegin);
+MarkMove(LineEnd);
+MarkMove(FileBegin);
+MarkMove(FileEnd);
+MarkMove(PageDown);
+MarkMove(PageUp);
+MarkMove(PageTop);
+MarkMove(PageBottom);
+MarkMove(LineUp);
+MarkMove(LineDown);
